#include "TSO.h"

//constructor
TSO::TSO(ProblemDescription *pd)
{
	this->pd = pd;
	alternative = pd->alternative;
	transModel = pd->transModel;
	transCap = pd->transCap;
	transCost1 = pd->transCost1;
	transCost2 = pd->transCost2;
	nNodes = pd->nNodes;
	nLinks = nNodes*nNodes;
	nSuppliers = pd->nSuppliers;
	totalFlows.resize(nNodes*nNodes);
	prevTotalFlows.resize(nNodes*nNodes);
	hasOptions.resize(nNodes*nNodes);
	mcCap.resize(nNodes*nNodes);
	for (int link = 0; link < nNodes*nNodes; link++)
	{
		mcCap[link] = transCost1[link] + transCost2[link] * transCap[link];
	}
	violations.assign(nNodes*nNodes, 0);
	resetStepLengths();

	//initQPSolver();
	if (transModel)
		initTransportationSolver(Supplier::env);
}
//initializes the TSO solver 
void TSO::initTransportationSolver(MSKenv_t env)
{
	network = pd->network;
	num_a = network.nLinks_a;
	num_paths = network.nPaths;
	num_rs = network.nRS;
	numVar = num_a + network.nPaths; //link flows, path flows
	numCons = num_a + network.nRS; // link-path flow, supply/demand
	maxDual = network.maxPathMC;
	optVal.resize(numVar, -69);
	prevOptVal.resize(numVar, -69);
	dualOptVal.resize(numVar, -69);
	transDemand.resize(num_rs);
	
	linkPrices.resize(num_a);
	prevLinkPrices.resize(num_a);
	newLinkPrices.resize(num_a);

	initMatrixes();

	if (env == NULL)
		r = MSK_makeenv(&env, NULL);

	if (r == MSK_RES_OK)
	{
		r = MSK_maketask(env, numCons, numVar, &task);
		if (r == MSK_RES_OK)
		{
			//r = MSK_linkfunctotaskstream(task, MSK_STREAM_LOG, NULL, Supplier::mosekPrint);

			if (r == MSK_RES_OK)
				r = MSK_appendcons(task, numCons);

			if (r == MSK_RES_OK)
				r = MSK_appendvars(task, numVar);
			
			// put linear link costs 
			for (int a = 0; a < num_a; a++)
			{
				if (r == MSK_RES_OK)
					r = MSK_putcj(task, a, transCost1_a[a]);
			}

			//// put linear capacity violation cost
			//for (size_t a = 0; a < num_a; a++)
			//{
			//	int as = num_a + num_paths + a;
			//	if (r == MSK_RES_OK)
			//		r = MSK_putcj(task, as, 2*maxDual);
			//}
			////set cost for demand violation (higher than max unit path cost) ---- OLD
			//for (size_t rs = 0; rs < network.nRS; rs++)
			//{
			//	if (r == MSK_RES_OK)
			//		r = MSK_putcj(task, num_a + num_paths + rs, 2 * network.maxPathMC);
			//}

			//link variable bounds - now unbinding!
			for (size_t a = 0; a < num_a; a++)
			{
				MSKboundkeye bkx = MSK_BK_RA;
				if (r == MSK_RES_OK)
					r = MSK_putvarbound(task,
					a,           /* Index of variable.*/
					bkx,      /* Bound key.*/
					0,      /* Numerical value of lower bound.*/
					+MSK_INFINITY);   //transCap_a[a]);     /* Numerical value of upper bound.*/
			}
			//other variable bounds (non-negative)
			for (size_t j = num_a; j < numVar; j++)
			{
				MSKboundkeye bkx = MSK_BK_LO; // > 0

				if (r == MSK_RES_OK)
					r = MSK_putvarbound(task,
					j,           /* Index of variable.*/
					bkx,      /* Bound key.*/
					0,      /* Numerical value of lower bound.*/
					+MSK_INFINITY);     /* Numerical value of upper bound.*/
			}
			//set constraints (note that bounds on rs-flow must be given from the suppliers)
			for (size_t cons = 0; cons < numCons; cons++)
			{
				if (r == MSK_RES_OK)
					r = MSK_putarow(task,
					cons,                 /* Variable (column) index.*/
					aptre[cons] - aptrb[cons], /* Number of non-zeros in column j.*/
					&asub[0] + aptrb[cons],     /* Pointer to row indexes of column j.*/
					&aval[0] + aptrb[cons]);    /* Pointer to Values of column j.*/
				if (r == MSK_RES_OK)
					r = MSK_putconbound(task,
					cons,           /* Index of constraint.*/
					MSK_BK_FX,      /* Bound key.*/
					0,      /* Numerical value of lower bound.*/
					0);     /* Numerical value of upper bound.*/
			}
			//put quadratic costs
			if (r == MSK_RES_OK)
				r = MSK_putqobj(task, numQnz, &qsub[0], &qsub[0], &qval[0]);
		}
	}
	if (r != MSK_RES_OK)
		throw "what?";
	MSK_putnaintparam(task, "MSK_IPAR_INTPNT_HOTSTART", MSK_INTPNT_HOTSTART_DUAL);
}

//initializes matrixes used by the TSO solver
void TSO::initMatrixes()
{
	//Variables: num_a link flows, then num_paths path flows, then num_rs capacity violations
	transCap_a.resize(0);
	transCost1_a.resize(0);
	transCost2_a.resize(0);
	qsub.resize(0);
	qval.resize(0);
	mc_a.resize(num_a);
	congRent_a.resize(num_a);
	newCongRent_a.resize(num_a);
	prevCongRent_a.resize(num_a);
	priceTarget.resize(num_rs);
	mcCap_a.resize(num_a);

	//add quadratic link cost
	numQnz = 0;
	for (size_t a = 0; a < num_a; a++)
	{
		transCap_a.push_back(transCap[network.link_for_a[a]]);
		transCost1_a.push_back(transCost1[network.link_for_a[a]]);
		transCost2_a.push_back(transCost2[network.link_for_a[a]]);
		mcCap_a[a] = transCost1_a[a] + 2 * transCap_a[a] * transCost2_a[a];
		qsub.push_back(a);
		qval.push_back(2 * transCost2[network.link_for_a[a]]);
		numQnz++;
	}
	////add quadratic capacity violation cost 
	//for (int a = 0; a < num_a; a++)
	//{
	//	int as = num_a + num_paths + a;
	//	qsub.push_back(as);
	//	qval.push_back(qCapCost);
	//	numQnz++;
	//}

	//Set A matrix
	numAnz = 0;
	aptrb.resize(numCons);
	aptre.resize(numCons);
	asub.resize(0);
	aval.resize(0);
	// Path to link flow constraint
	for (size_t a = 0; a < num_a; a++)
	{
		asub.push_back(a);
		aval.push_back(1);
		//asub.push_back(num_a + num_paths + a); <-Slack variables
		//aval.push_back(1);
		int rownz = 1;
		for each (Path p in network.paths_for_a[a])
		{
			asub.push_back(num_a + p.i);
			aval.push_back(-1);
			rownz++;
		}
		numAnz += rownz;
		if (a == 0)
			aptrb[a] = 0;
		else
			aptrb[a] = aptre[a - 1];
		aptre[a] = aptrb[a] + rownz;
	}
	//Demand constraint
	int rs = 0;
	for (size_t link = 0; link < nLinks; link++)
	{
		if (!network.hasPath[link])
			continue;
		int coni = num_a + rs;
		int rownz = 0;
		for each (Path p in network.paths[link])
		{
			asub.push_back(num_a + p.i);
			aval.push_back(1);
			rownz++;
		}
		//asub.push_back(num_a + num_paths + rs); <- From when we used slack on trans demand
		//aval.push_back(1);
		//rownz++;
		numAnz += rownz;
		aptrb[coni] = aptre[coni - 1];
		aptre[coni] = aptrb[coni] + rownz;
		rs++;
	}
}

//adjust transportation prices (This is called for all Alternatives, 
//	but only alt 1 adjust here, the others call the methods below
bool TSO::adjustPrices(DecisionVariables* decisions, Prices *prices,
	double capacitySlack, double clearingSlack, double dualCapSlack)
{

	if (transModel)
	{
		return adjustPricesTransModel(decisions, prices, capacitySlack, clearingSlack);
	}
	double gapCap, gapMrMc, mr, mc, flow, prevFlow, prevMc, prevMr, cap, curPrice, prevPrice, p_mcCap;
	bool notHappy = false;

	setTotalFlows(decisions);

	bool hasChangedFlows = checkCapViolations(decisions, prices, capacitySlack, dualCapSlack);

	if (hasChangedFlows)
		totalFlows = temptotalFlows;

	for (int link = 0; link < nNodes*nNodes; link++)
	{
		if (link % nNodes == link / nNodes)
			continue;
		flow = totalFlows[link];
		prevFlow = prevTotalFlows[link];
		cap = transCap[link];
		gapCap = flow - cap;
		curPrice = prices->congestion[link];
		prevPrice = prices->ppcon[link];
		mc = transCost1[link] + flow * 2 * transCost2[link];
		prevMc = transCost1[link] + prevFlow * 2 * transCost2[link];
		mr = prices->congestion[link];
		prevMr = prevPrice;
		gapMrMc = mc - mr;
		prices->congRent[link] = flow;

		if (violations[link])
		{
			if (curPrice > prevPrice) //(prevViolations[link]) //
				prices->congestion[link] += step2*(curPrice - prevPrice);
			else if (curPrice < prevPrice)
				prices->congestion[link] = step1*(curPrice + prevPrice);
			else
				prices->congestion[link] += step_c * mcCap[link];
		}
		else if (abs(gapCap) <= capacitySlack)
		{
			if (curPrice < mcCap[link] - clearingSlack)
			{
				if (curPrice > prevPrice)
					prices->congestion[link] += step2*(curPrice - prevPrice);
				else if (curPrice < prevPrice)
					prices->congestion[link] = step1*(curPrice + prevPrice);
				else
					prices->congestion[link] += step * (mcCap[link] - curPrice);
			}
			else//Happy
				continue;
		}
		else // (flowA < capA - capacitySlack)
		{
			if (abs(gapMrMc) <= clearingSlack)
			{
				//Happy
				continue;
			}
			else if (curPrice == prevPrice) // || prevViolations[link])
			{
				//take a step closer to marginal cost
				prices->congestion[link] += step * gapMrMc;
			}
			else if (curPrice < mc && curPrice > prevPrice || curPrice > mc && curPrice < prevPrice)
				prices->congestion[link] += step2*(curPrice - prevPrice); //continue in same direction with bigger step
			else if (curPrice > mc && curPrice > prevPrice || curPrice < mc && curPrice < prevPrice)
				prices->congestion[link] = step1*(curPrice + prevPrice); //smaller step in opposite direction
			else
				throw "what";
		}

		notHappy = true;
		if (prices->congestion[link] < transCost1[link])
			prices->congestion[link] = transCost1[link];
	}
	return notHappy;
}

// Adjust prices for Alt 2 and 3
bool TSO::adjustPricesTransModel(DecisionVariables* decisions, Prices *prices,
	double capacitySlack, double clearingSlack, double dualCapSlack)
{
	//Set target flows from supplier decisions
	int rs = 0;
	for (size_t i = 0; i < nNodes; i++)
	{
		for (size_t j = 0; j < nNodes; j++)
		{
			int link = i*nNodes + j;
			if (i == j || !network.hasPath[link])
				continue;
			double sumrs = 0;
			for (size_t sup = 0; sup < nSuppliers; sup++)
			{
				sumrs += decisions[sup].flow[link];
			}
			int coni = num_a + rs;
			transDemand[rs] = sumrs;
			rs++;
			r = MSK_putconbound(task,
				coni,           /* Index of constraint.*/
				MSK_BK_FX,      /* Bound key.*/
				sumrs,      /* Numerical value of lower bound.*/
				sumrs);     /* Numerical value of upper bound.*/
			if (r != MSK_RES_OK)
				throw "what?";
		}
	}

	//Set transportation cost according to congestion rents
	for (int a = 0; a < num_a; a++)
	{
		r = MSK_putcj(task, a, transCost1_a[a] + congRent_a[a]);
		if (r == !MSK_RES_OK)
			throw "what?";
	}

	//solve transportation problem
	runMosek();

	//Set rs marginal costs
	for (int rs = 0; rs < num_rs; rs++)
	{
		double dem;
		priceTarget[rs] = dualOptVal[num_a + rs];
		if (dualOptVal[num_a + rs] == 0)
		{
			dem = transDemand[rs];
			if (dem > 1)
				rs = rs;
		}
	}
	bool ret;
	if (alternative == 2)
		ret = adjustPricesAlt2(prices, capacitySlack, clearingSlack, dualCapSlack);
	else if (alternative == 3)
		ret = adjustPricesAlt3(prices, capacitySlack, clearingSlack, dualCapSlack);
	else
		throw "what?";

	//prices->linkP.resize(nLinks);
	//prices->congRent.resize(nLinks);
	for (int a = 0; a < num_a; a++)
	{
		int l = network.link_for_a[a];
		prices->congRent[l] = optVal[a];
		prices->linkP[l] = linkPrices[a];
	}
	//for (int rs = 0; rs < num_rs; rs++)
	//{
	//	prices->linkP[network.link_for_rs[rs]] = transDemand[rs];
	//}
	return ret;
	//double mc = mc_rs[rs];
	//double MC_d = dualOptVal[num_a + rs];
	//double mcdgap = mc - MC_d;
	//if (isViolated[rs])
	//{
	//	double v = optVal[num_a + num_paths + rs];
	//	double d = transDemand[rs];
	//	double g = mc* v/ d;
	//	increasePrice(prices, rsi, g);
	//}
	//else if (isOnCap_rs[rs])
	//{
	//	if (mr < mc)
	//		increasePrice(prices, rsi, mc);
	//}
	//else
	//{
	//	double MCMRGap = mc - mr;
	//	double demand = transDemand[rs];
	//	if (demand < 100*solverSlack && mr <= mc + clearingSlack)
	//		continue;
	//	if (abs(MCMRGap) > clearingSlack)
	//		bringPriceCloserToMC(prices, rsi, MCMRGap);
	//}
}

//Alt 2
bool TSO::adjustPricesAlt2(Prices *prices,
	double capacitySlack, double clearingSlack, double dualCapSlack)
{
	////find link marginal costs
	//for (int a = 0; a < num_a; a++)
	//{
	//	int l = network.link_for_a[a];
	//	mc_a[a] = transCost1_a[a] + 2 * optVal[a] * transCost2_a[a];
	//}

	newLinkPrices = linkPrices;
	changedPrice = false;
	//adjust link prices
	for (int a = 0; a < num_a; a++)
	{
		double flowA = optVal[a];
		//double prevFlowA = prevOptVal[a];
		double capA = transCap_a[a];
		double priceA = linkPrices[a];
		double prevPriceA = prevLinkPrices[a];
		double priceChange = 0;
		double congA = congRent_a[a];
		double prevCongA = prevCongRent_a[a];
		double minPrice = transCost1_a[a];
		double mcA = minPrice + 2 * flowA * transCost2_a[a];
		double mcCapA = mcCap_a[a];
		double gapMrMc = mcA - priceA;
		double changeCong = 0;
		if (flowA > capA * (1 + capacitySlack))
		{
			if (priceA > prevPriceA)
				priceChange = step2*(priceA - prevPriceA);
			else if (priceA < prevPriceA)
				priceChange = step1*(prevPriceA - priceA); //smaller step in opposite direction
			else
				priceChange = step * mcA * (flowA - capA) / capA;
		}
		else if (flowA >= capA * (1 - capacitySlack))
		{
			if (priceA < mcCap_a[a] - clearingSlack)
			{
				if (priceA > prevPriceA)
					priceChange = step2*(priceA - prevPriceA);
				else if (priceA < prevPriceA)
					priceChange = step1*(prevPriceA - priceA); //smaller step in opposite direction
				else
					priceChange = step * (mcCap_a[a] - priceA);
			}
			else//Happy
				continue;
		}
		else // (flowA < capA - capacitySlack)
		{
			if (abs(priceA - mcA) <= clearingSlack)
			{
				//Happy
				continue;
			}
			else if (priceA == prevPriceA)// || prevViolations[link])
			{
				//take a step closer to marginal cost
				priceChange = step*gapMrMc;
			}
			else if (priceA < mcA && priceA > prevPriceA || priceA > mcA && priceA < prevPriceA)
				priceChange = step2*(priceA - prevPriceA); //bigger step in same direction
			else if (priceA > mcA && priceA > prevPriceA || priceA < mcA && priceA < prevPriceA)
				priceChange = step1*(prevPriceA - priceA); //smaller step in opposite direction
			else
				throw "what";
		}
		//change price
		if (priceChange == 0)
			throw "what";
		double newPrice = priceA + priceChange;
		if (newPrice < minPrice)
			newPrice = minPrice;
		newLinkPrices[a] = newPrice;
		congRent_a[a] = fmax(newPrice - mcCapA, 0);
		changedPrice = true;
	}

	if (changedPrice == false)
		return false;

	prevLinkPrices = linkPrices;
	linkPrices = newLinkPrices;

	//find SD prices by finding shortest paths
	findShortestPaths();

	for (int rs = 0; rs < num_rs; rs++)
	{
		int rsi = network.link_for_rs[rs];
		prices->congestion[rsi] = spCost[rsi];
	}

	return true;
}

//Alt 3
bool TSO::adjustPricesAlt3(Prices *prices,
	double capacitySlack, double clearingSlack, double dualCapSlack)
{
	//find targets for duals == 0
	for (int a = 0; a < num_a; a++)
		linkPrices[a] = transCost1_a[a] + congRent_a[a] + 2 * transCost2_a[a] * optVal[a];
	findShortestPaths();
	//set price target equal to dual of demand constraint
	for (int rs = 0; rs < num_rs; rs++)
	{
		priceTarget[rs] = dualOptVal[num_a + rs];
		if (dualOptVal[num_a + rs] == 0)
			priceTarget[rs] = spCost[network.link_for_rs[rs]];
	}
	//adjust prices towards target
	changedPrice = false;
	bool c = false;
	for (size_t rs = 0; rs < num_rs; rs++)
	{
		int rsi = network.link_for_rs[rs];
		double mr = prices->congestion[rsi];
		//if (dualOptVal[num_a + rs] == 0)
		//	priceTarget[rs] = spCost[rsi];
		//double gap = fmax(priceTarget[rs], network.minPrices[rsi]) - mr;
		//double gap = fmax(priceTarget[rs], network.minPrices[rsi]) - mr;
		double gap = spCost[rsi] - mr;
		prices->linkP[rsi] = priceTarget[rs];
		if (abs(gap) > clearingSlack)
		{
			c = true;
			bringPriceCloserToMC(prices, rsi, gap); //prices->congestion[rsi] += gap * 0.1;
		}
	}

	//Adjust congestion rents
	bool hasChangedCong = false;
	for (int a = 0; a < num_a; a++)
	{
		double flowA = optVal[a];
		double capA = transCap_a[a];
		double congA = congRent_a[a];
		double prevCongA = prevCongRent_a[a];
		double mcA = transCost1_a[a] + 2 * flowA * transCost2_a[a];
		double changeCong = 0;
		if (flowA < capA * (1 - capacitySlack))
		{
			if (congA == 0)
			{
				prevCongRent_a[a] = congA;
				continue;
			}
			else
			{ //adjust down
				if (congA <= clearingSlack)
					changeCong = -congA;
				else if (congA < prevCongA)
					changeCong = (congA - prevCongA) * step2;
				else if (prevCongA < congA)
					changeCong = (prevCongA - congA) * step1;
				else
					changeCong = step * mcA * (flowA - capA) / capA; //step * 2 * (flowA - capA) / capA * transCost2_a[a]consider using MC between flow and cap
			}
		}
		else if (flowA <= capA * (1 + capacitySlack))
		{
			prevCongRent_a[a] = congA;
			continue;
		}
		else
		{ //adjust up
			if (congA > prevCongA)
				changeCong = (congA - prevCongA) * step2;
			else if (prevCongA > congA)
				changeCong = (prevCongA - congA) * step1;
			else
				changeCong = step * mcA * (flowA - capA) / capA; //mcA*step consider using MC between flow and cap
		}
		prevCongRent_a[a] = congA;
		if (changeCong != 0)
		{//adjust target prices according to change in congestion rate
			hasChangedCong = true;
			hasAdjustedCong_rs.assign(num_rs, false);
			changeCong = fmax(-congA, changeCong); // Avoiding negative congestion rents
			congRent_a[a] += changeCong;
			for each (Path p in network.paths_for_a[a])
			{
				double pFlow = optVal[num_a + p.i];
				if (pFlow > solverSlack)
				{
					if (pFlow < 1)
						hasChangedCong = hasChangedCong;
					int link = p.nodes[0] * nNodes + p.nodes.back();
					int rs = network.rs_for_link[link];
					if (!hasAdjustedCong_rs[rs])
					{
						priceTarget[rs] += changeCong;
						prices->congestion[link] += changeCong;
						hasAdjustedCong_rs[rs] = true;
					}
				}
			}
		}
	}




	return c || hasChangedCong;
}

//enforcing capacities on supplier bids (alt 1)
bool TSO::checkCapViolations(DecisionVariables* decisions, Prices *prices,
	double capacitySlack, double dualSlackp)
{
	prevViolations = violations;
	violations.assign(nLinks, 0);
	hadOptions = hasOptions;
	hasOptions.assign(nNodes*nNodes, 0);
	capViolDuals.resize(nNodes*nNodes);
	std::set<int> violators;
	double dualSlack = fmax(dualSlackp, 1E-10);
	doubleVector violatorFlows, supTransCap(nNodes*nNodes, 0);
	for (int link = 0; link < nNodes*nNodes; link++)
	{
		if (transCap[link]>0)
			supTransCap[link] = std::numeric_limits<double>::max();
	}
	std::vector<doubleVector> tempTransCaps(nSuppliers, supTransCap);
	bool hasMovedFlow = false, movedLast = true, hasViolations = false;
	double totFlow, supFlow;
	DecisionVariables dec(nNodes);
	std::vector<DecisionVariables> decs(nSuppliers, dec);
	for (int sup = 0; sup < nSuppliers; sup++)
		decs[sup] = decisions[sup];
	int countRestrictions = 0;
	// capacity pass
	while (movedLast)
	{
		movedLast = false;
		setTotalFlows(decisions, true);
		for (int link = 0; link < nNodes*nNodes; link++)
		{
			totFlow = temptotalFlows[link];
			double viol = totFlow - transCap[link];
			if (transCap[link] > 0 && totFlow > transCap[link] + capacitySlack)
			{
				countRestrictions++;
				if (countRestrictions > nNodes*nNodes)
					throw "should not happen";
				for (int sup = 0; sup < nSuppliers; sup++)
				{
					supFlow = decisions[sup].flow[link];
					if (supFlow > 0)
					{
						hasMovedFlow = true;
						movedLast = true;
						double temp = supFlow / totFlow * transCap[link];
						tempTransCaps[sup][link] = supFlow / totFlow * transCap[link];
						suppliers[sup].decide(prices, &decs[sup], &tempTransCaps[sup]);
						decideCalls++;
						double dual = decs[sup].duals[2 * nNodes + link];
						if (dual >= -dualSlack)
						{
							hasOptions[link] = 1;
							decisions[sup] = decs[sup];
						}
						else
						{
							decisions[sup] = decs[sup];
							hasViolations = true;
							violators.insert(sup);
							violations[link] = 1;
							capViolDuals[link] = fmax(-dual, capViolDuals[link]);
						}
					}
				}
			}
			if (movedLast)
				break;
		}
	}
	return hasMovedFlow || hasViolations;
}

//floyd warshall for finding shortest paths using the linkprices
void TSO::findShortestPaths()
{
	double max = std::numeric_limits<double>::max();
	spCost.assign(nLinks, max);
	spNext.assign(nLinks, -1);
	for (int a = 0; a < num_a; a++)
	{
		int l = network.link_for_a[a];
		spCost[l] = linkPrices[a]; //mc_a[a] + congRent_a[a];
		spNext[l] = l % nNodes;
	}
	for (int k = 0; k < nNodes; k++)
	{
		for (int i = 0; i < nNodes; i++)
		{
			for (int j = 0; j < nNodes; j++)
			{
				double ik = spCost[i*nNodes + k];
				double kj = spCost[k*nNodes + j];
				double sumikj = ik + kj;
				if (ik != max && kj != max && sumikj < spCost[i*nNodes + j])
				{
					spCost[i*nNodes + j] = sumikj;
					spNext[i*nNodes + j] = spNext[i*nNodes + k];
				}
			}
		}
	}
}

//Method for solving the TSO problem (Alt 2 and 3)
void TSO::runMosek()
{
	tsoProblemCalls++;
	//store previous solution
	prevOptVal = optVal;
	MSKrescodee trmcode;

	/* Run optimizer */
	r = MSK_optimizetrm(task, &trmcode);

	if (r == MSK_RES_OK)
	{
		MSKsolstae solsta;
		int j;

		MSK_getsolsta(task, MSK_SOL_ITR, &solsta);

		switch (solsta)
		{
		case MSK_SOL_STA_OPTIMAL:
		case MSK_SOL_STA_NEAR_OPTIMAL:
			MSK_getxx(task,
				MSK_SOL_ITR,    /* Request the interior solution. */
				&optVal[0]);

			MSK_getslc(task,
				MSK_SOL_ITR, // interior (?) dual
				&dualOptVal[0]);

			break;
		case MSK_SOL_STA_DUAL_INFEAS_CER:
		case MSK_SOL_STA_PRIM_INFEAS_CER:
		case MSK_SOL_STA_NEAR_DUAL_INFEAS_CER:
		case MSK_SOL_STA_NEAR_PRIM_INFEAS_CER:
			printf("Primal or dual infeasibility certificate found.\n");
			break;

		case MSK_SOL_STA_UNKNOWN:
			printf("The status of the solution could not be determined.\n");
			break;
		default:
			printf("Other solution status.");
			break;
		}
	}
	else
	{
		printf("Error while optimizing.\n");
	}
}

// ----------- LESS IMPORTANT METHODS BELOW ----------- 

TSO::~TSO()
{
}

void TSO::setInitialPrices(Prices *prices)
{
	if (transModel)
	{
		for (size_t i = 0; i < nLinks; i++)
		{
			prices->congestion[i] = network.minPrices[i];
		}
		prices->saveFlowPricesAsPrevious();
		prices->saveFlowPricesAsPrevious();
		for (int a = 0; a < num_a; a++)
		{
			linkPrices[a] = prevLinkPrices[a] = transCost1_a[a];
		}
	}
	else
		TSO::setInitialPrices(prices, initPriceK, pd);
}
void TSO::setInitialPrices(Prices *prices, double initPriceK, ProblemDescription *pd)
{
	int nNodes = prices->nNodes;
	for (int i = 0; i < nNodes; i++)
	{
		for (int j = 0; j < nNodes; j++)
		{
			if (i == j)
				prices->congestion[i*nNodes + j] = 0;
			else
				prices->congestion[i*nNodes + j] = pd->transCost1[i*nNodes + j] + initPriceK * 2 * pd->transCap[i*nNodes + j] * pd->transCost2[i*nNodes + j];
		}
	}
	prices->saveFlowPricesAsPrevious();
	prices->saveFlowPricesAsPrevious();
}

void TSO::resetStepLengths()
{
	step = init_step;
	step_c = init_step_c;
	prev_sumgaps = 99999999999999;
	prev_sumgaps_c = 99999999999999;
}

void TSO::setStepLengthFromIteration(int iteration)
{
	int shift = 1;
	step = init_step * shift / (shift + iteration-1);
	step_c = init_step_c * shift / (shift + iteration-1);
}

void TSO::reduceStep0(double changeFactor)
{
	//step = step * changeFactor;
	step_c = step_c * changeFactor;
}

void TSO::setTotalFlows(DecisionVariables *decisions, bool temp)
{
	if (temp)
		temptotalFlows.assign(nNodes*nNodes, 0);
	else
	{
		ppTotalFlows = prevTotalFlows;
		prevTotalFlows = totalFlows;
		totalFlows.assign(nNodes*nNodes, 0);
	}
	
	for (int k = 0; k < nSuppliers; k++)
	{
		for (int i = 0; i < nNodes; i++)
		{
			for (int j = 0; j < nNodes; j++)
			{
				if (temp)
					temptotalFlows[i*nNodes + j] += decisions[k].flow[i*nNodes + j];
				else
					totalFlows[i*nNodes + j] += decisions[k].flow[i*nNodes + j];
			}
		}
	}
}

void TSO::printCapacityViolations()
{
	printf("\nTSO Capacity violations:\n\t");
	for each (auto link in violations)
	{
		printf("\t%i", link);
	}
}

//void TSO::initQPSolver()
//{
//	nWSR = _nWSR;
//	nVariables = nNodes*nNodes;
//	first = true;
//	qProblem = new qpOASES::QProblemB(nVariables);
//	//options.setToFast();
//	options.printLevel = qpOASES::PrintLevel(0);
//	qProblem->setOptions(options);
//
//	H = new double[nVariables*nVariables];
//	g = new double[nVariables];
//	lb = new double[nVariables];
//	ub = new double[nVariables];
//	optVal = new double[nVariables];
//	dualOptVal = new double[nVariables];
//
//	memset(H, 0, nVariables*nVariables*sizeof(double));
//
//	for (int link = 0; link < nVariables; link++)
//	{
//		lb[link] = 0;
//		ub[link] = transCap[link];
//		H[link*nVariables + link] = transCost2[link];
//	}
//}

void TSO::printTotalFlows()
{
	printf("Total Flows:\t");
	for (int i = 0; i < nNodes; i++)
	{
		for (int j = 0; j < nNodes; j++)
		{
			printf("\t%4.4f ", totalFlows[nNodes*i + j]);
		}
		if (i < nNodes - 1)
			printf("\n\t\t");
	}
}



void TSO::bringPriceCloserToMC(Prices *prices, int link, double gap)
{
	if (gap > 0)
		increasePrice(prices, link, gap);
	else
		decreasePrice(prices, link, gap);
}

void TSO::increasePrice(Prices *prices, int link, double gap)
{
	changedPrice = true;
	double prevPrice = prices->ppcon[link];
	double curPrice = prices->congestion[link];
	if (prevPrice == curPrice)
		prices->congestion[link] += step * gap;
	else if (prevPrice < curPrice)
		prices->congestion[link] += step2 * (curPrice - prevPrice);
	else //prevprice > curprice
		prices->congestion[link] += step1 * (prevPrice - curPrice);
}

void TSO::decreasePrice(Prices *prices, int link, double gap)
{
	changedPrice = true;
	double prevPrice = prices->ppcon[link];
	double curPrice = prices->congestion[link];
	if (prevPrice == curPrice)
		prices->congestion[link] += step * gap;
	else if (prevPrice > curPrice)
		prices->congestion[link] += step2 * (curPrice - prevPrice);
	else //prevprice < curprice
		prices->congestion[link] += step1 * (prevPrice - curPrice);
	prices->congestion[link] = fmax(network.minPrices[link], prices->congestion[link]);
}


void TSO::reset()
{
	resetStepLengths();
	decideCalls = 0;
	tsoProblemCalls = 0;
	nWSR = _nWSR;
	first = true;
}


//Tabu search wrappers below, not in use

TabuTSO::TabuTSO(ProblemDescription *pd, int listLength) : TSO(pd)
{
	this->tabuListLength = listLength;
	newestElement = listLength - 1;
	linkPriceHistory.resize(listLength);
	for each (vector<double> log in linkPriceHistory)
		log.resize(pd->nNodes*pd->nNodes);
}

TabuTSO::~TabuTSO()
{
}

bool TabuTSO::adjustPrices(DecisionVariables* decisions, Prices *prices, double capacitySlack, double clearingSlack, double dualCapSlack)
{
	// Initial calculations
	setTotalFlows(decisions);
	if (checkCapViolations(decisions, prices, capacitySlack, dualCapSlack))
		totalFlows = temptotalFlows;
	bool notHappy = false;

	// Instead of changing the prices directly, the TabuTSO makes a suggestion which will be approved or thrown away/modified by the TABU-search
	vector<double> newSuggestion;
	newSuggestion.resize(nNodes*nNodes);

	// Iterate over all links
	for (int link = 0; link < nNodes*nNodes; link++)
	{
		// Skip if the link is to and from the same node
		if (link % nNodes == link / nNodes) continue;
		// Calculate some stats for current link
		double
			flow = totalFlows[link],
			prevFlow = prevTotalFlows[link],
			cap = transCap[link],
			gapCap = flow - cap,
			curPrice = prices->congestion[link],
			prevPrice = prices->ppcon[link],
			mc = transCost1[link] + flow * 2 * transCost2[link],
			prevMc = transCost1[link] + prevFlow * 2 * transCost2[link],
			mr = prices->congestion[link],
			prevMr = prevPrice,
			gapMrMc = mc - mr
			;

		// If the link has violations, suggest to increase the price
		if (violations[link])
		{
			if (prevViolations[link])
				newSuggestion[link] = prices->congestion[link] + step2*(curPrice - prevPrice);
			else
				newSuggestion[link] = prices->congestion[link] + step*mc;
		}
		// If TODO: elaborate
		else if (abs(gapCap) <= 1000 * capacitySlack)
		{
			if (mr < mcCap[link])
				newSuggestion[link] = prices->congestion[link] + step*mcCap[link];
			else
				continue;
		}
		// If the flow is below capacity
		else if (flow < cap - capacitySlack)
		{
			if (abs(gapMrMc) <= clearingSlack)
				continue;
			else if (curPrice == prevPrice)
				newSuggestion[link] = prices->congestion[link] + step*gapMrMc;
			else if (mr < mc)
			{
				// Adjust the price on the link according to mr = mc
				if (prevMr < prevMc - clearingSlack)
				{
					//was too low in last iteration as well
					//take a longer step
					if (curPrice < prevPrice)
						notHappy = true;
					newSuggestion[link] = prices->congestion[link] + step2*(curPrice - prevPrice);
				}
				else if (prevMr > prevMc + clearingSlack)
				{
					//too low now and too high earlier - meet in the middle
					if (curPrice > prevPrice)
						notHappy = true;
					newSuggestion[link] = step1*(curPrice + prevPrice);
				}
				else
				{
					//take a step closer towards marginal cost
					newSuggestion[link] = prices->congestion[link] + step*gapMrMc;
				}
			}
			else if (mc < mr)
			{
				//Price is too high, 
				if (prevMr > prevMc + clearingSlack)// && curPrice < prevPrice)
				{
					//was too high in last iteration as well
					//take a longer step down
					if (prevViolations[link])
						newSuggestion[link] = prices->congestion[link] + step*gapMrMc;
					else
					{
						newSuggestion[link] = prices->congestion[link] + step2*(curPrice - prevPrice);
						if (curPrice > prevPrice)
							notHappy = true;
					}
				}
				else if (prevMr < prevMc - clearingSlack)
				{
					//too high now,too low earlier - meet in the middle
					if (curPrice < prevPrice)
						notHappy = true;
					newSuggestion[link] = step1*(curPrice + prevPrice);
				}
				else
				{
					//take a step closer to marginal cost
					newSuggestion[link] = prices->congestion[link] + step*gapMrMc;
				}
			}
			else
			{
				//bring price closer to marginal cost
				newSuggestion[link] = prices->congestion[link] + step*gapMrMc;
			}
		}
		else
		{
			// higher price to meet capacity restriction
			double p_mcCap = mcCap[link];
			if (curPrice < p_mcCap)
				newSuggestion[link] = p_mcCap;
		}

		notHappy = true;
		if (prices->congestion[link] < transCost1[link])
			newSuggestion[link] = transCost1[link];

	}

	// Assert if the new prices are ok to implement
	if (approveSuggestion(newSuggestion))
	{
		// If yes, then implement them
		for (int link = 0; link < nNodes*nNodes; link++)
			prices->congestion[link] = newSuggestion[link];
		// Add them to the tabu list
		addToHistory(newSuggestion);
	}
	else
	{
		// If not, what do we do? New suggestion.. 
	}

	return notHappy;

}

bool TabuTSO::approveSuggestion(vector<double> suggestion)
{
	// TODO: Implement. Remember that the aspiration criteria should also come into play here
	return true;
}

void TabuTSO::addToHistory(vector<double> prices)
{
	// Increase the index by 1, to put the prices in the correct position
	newestElement = (newestElement + 1) % tabuListLength;
	// Insert the prices
	linkPriceHistory[newestElement] = prices;
}


//old
//void TSO::decide(const Prices * const prices, DecisionVariables* const decision)
//{
//	for (int link = 0; link < nVariables; link++)
//	{
//		double gl = transCost1[link] - prices->congestion[link];
//		g[link] = gl;
//	}
//	qpOASES::returnValue res;
//	nWSR = _nWSR;
//	if (first)
//	{
//		res = qProblem->init(H, g, lb, ub, nWSR);
//		first = false;
//	}
//	else
//	{
//		res = qProblem->hotstart(g, lb, ub, nWSR);
//	}
//
//	//check solver status
//	if (res != qpOASES::SUCCESSFUL_RETURN)
//	{
//		throw "Suppliers problem not solved!";
//	}
//	//get optimal values
//	qProblem->getPrimalSolution(optVal);
//	qProblem->getDualSolution(dualOptVal);
//
//	decision->tsoflows = optVal;
//}
//bool TSO::setNewCongestionPrices(DecisionVariables* decisions, Prices *prices, double clearingSlack)
//{
//	bool hasGap = false;
//	double mr, mc, gap, sumgap = 0;
//	int link = 0;
//
//	setTotalFlows(decisions);
//
//	if (printGaps) 
//		printf("\n\tTSO gaps:");
//	for (int i = 0; i < nNodes; i++)
//	{
//		for (int j = 0; j < nNodes; j++)
//		{
//			link = i*nNodes + j;
//			mc = transCost1[link] + totalFlows[link] * 2 * transCost2[link];
//			mr = prices->congestion[link];
//			gap = mc - mr;
//			if (abs(gap) <= clearingSlack || i == j)
//			{
//				/*if (printGaps)
//					printf("\t0.000");*/
//			}
//			else if (abs(gap) > clearingSlack)
//			{
//				if (printGaps)
//					printf("\t%i:%2.2f", link, gap); //printf("\t%4.4f", gap);
//				sumgap += abs(gap);
//				prices->congestion[i*nNodes + j] += step*(mc - mr); 
//				hasGap = true;
//			}
//		}
//	}
//	if (printSumGaps)
//	{
//		printf("\nSum gaps: %4.4f", sumgap); //printf("\t%4.4f", gap);
//	}
//	if (sumgap > prev_sumgaps)
//	{
//		step *= 0.8;
//	}
//	prev_sumgaps = sumgap;
//	return hasGap;
//}
//
//bool TSO::findCapacityViolations(DecisionVariables *decisions, double capacitySlack)
//{
//	bool hasViolations = false;
//	violations.resize(0);
//
//	//setTotalFlows(decisions);
//
//	int link = 0;
//	for (int i = 0; i < nNodes; i++)
//	{
//		for (int j = 0; j < nNodes; j++)
//		{
//			link = i*nNodes + j;
//			if (i != j && totalFlows[link] > transCap[link] + capacitySlack)
//			{
//				hasViolations = true;
//				violations.push_back(link);
//			}
//		}
//	}
//	if (printCapAdjust)
//		printCapacityViolations();
//	return hasViolations;
//}
//
//bool TSO::findNewCapacityViolations(DecisionVariables *decisions, double capacitySlack)
//{
//	if (violations.size() == 0)
//		return findCapacityViolations(decisions, capacitySlack);
//	bool hasViolations = false, afterLast = false;
//	//std::vector<int> newViols, allViols;
//	std::vector<int>::iterator it = violations.begin();
//	setTotalFlows(decisions);
//
//	int link = 0, orgViolation = violations.front(), orgi = 0;
//
//	for (int i = 0; i < nNodes; i++)
//	{
//		for (int j = 0; j < nNodes; j++)
//		{
//			link = i*nNodes + j;
//			if (link == orgViolation){
//				if (++orgi < violations.size()){
//					orgViolation = violations[orgi];
//					it = violations.begin() + orgi;
//				}
//				else
//					afterLast = true;
//			}
//			else if (i != j && totalFlows[link] > transCap[link] + capacitySlack)
//			{
//				hasViolations = true;
//				if (afterLast)
//					violations.push_back(link);
//				else
//					violations.insert(it, link); 
//			}
//		}
//	}
//	return hasViolations;
//}
//
//bool TSO::adjustForCapacityRestrictions(DecisionVariables* decisions, Prices *prices,
//	double capacitySlack, double clearingSlack)
//{
//	bool hasGap = false;
//	double mr, mc, gap,cap, sumgap = 0, sumgap_c = 0;
//	int link, violation  = (violations.size() > 0 ? violations[0] : -1), violationi=0;
//
//	setTotalFlows(decisions);
//
//	if (printCapAdjust)
//		printf("\n\tCap gaps:");
//	for (int i = 0; i < nNodes; i++)
//	{
//		for (int j = 0; j < nNodes; j++)
//		{
//			link = i*nNodes + j;
//			if (link == violation)
//			{
//				gap = totalFlows[link] - transCap[link];
//				if (abs(gap) > capacitySlack)
//				{
//					double change = step_c*gap / transCap[link] * prices->congestion[link]; //+= 0.5*gap;// 
//					double price = prices->congestion[link];
//					prices->congestion[link] += change; 
//					sumgap_c += abs(gap);
//					hasGap = true;
//				}
//				if (printCapAdjust)
//				{
//					printf("\t%f", gap);
//				}
//				if (++violationi < violations.size())
//					violation = violations[violationi];
//				break;
//			}
//			else if (transCap[link] > 0 && abs(totalFlows[link] - transCap[link]) <= capacitySlack)
//			{
//				break;
//			}
//			mc = transCost1[link] + totalFlows[link] * 2 * transCost2[link];
//			mr = prices->congestion[link];
//			gap = mc - mr;
//			if (abs(gap) <= clearingSlack || i == j)
//			{
//				if (printGaps)
//					printf("\t0.000");
//			}
//			else if (abs(gap) > clearingSlack)
//			{
//				if (printGaps)
//					printf("\t%4.4f", gap);
//				prices->congestion[link] += step*gap;
//				sumgap += abs(gap);
//				hasGap = true;
//			}
//		}
//	}
//	if (sumgap_c > 1.1*prev_sumgaps_c)
//	{
//		step_c *= 0.9;
//	}
//	prev_sumgaps_c = sumgap_c;
//	if (sumgap > 1.1*prev_sumgaps)
//	{
//		step *= 0.9;
//	}
//	prev_sumgaps = sumgap;
//	return hasGap;
//}
